function [E,X2,RECON,SYNTH] = P_6(X1,BLOCKS,E1)
% [E,X2,RECON,SYNTH] = P_6(X1,BLOCKS,E1)
%
%       E:      prediction error signal
%
%       X2:     predicted speech
%
%       RECON:  reconstructed speech using prediction error
%
%       SYNTH:  synthesized speech using excitation vector
%
%       X1:     original speech signal
%
%       BLOCKS: # of full 20ms segments in original speech signal
% 		(sampled at 8 kHz)
%
%       E1:     periodic impulse train excitation of pitch period P.
%               P does not appear in the program.  It is used when you generate
%               E1. P should be in the range 50-300Hz
%
% In Lab 6, you need to modify this file to generate E, Recon, and
% Synth.  X2 is generated as an example.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       ADDITIONAL PARAMETERS:
%
%       NP:     order of the prediction filter
%       N:      # of samples in one 20ms segment which has been sampled a
%               8000Hz
%       G:      Gain of the discrete-time prediction filter
%       ALPH:   vector of prediction filter coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NP = 10;  	% Order of the prediction filter
N = 160;  	% Number of samples in each block

% Design of Hamming window for use in determining 
% the autocorrelation coefficients

W=hamming(N);
W=W';


%
%   Block processing starts
%


for BLOCK = 1:BLOCKS

  if BLOCK / 10 == floor(BLOCK/10),
    fprintf('Up to block %d.\n',BLOCK);
  end

  for I=1:N
    X(I) = X1(I+(BLOCK-1)*N);    % Segmentation of input sequence
  end

  X = X.*W;   % Windows input data
%
% Calculation of the autocorrelation coefficients
%

  for I = 1:(NP+1)
       R(I) = (X(1:N-I+1)*X(I:N)')./N;
  end

%
% Calculation of the prediction filter coefficients using the Toeplitz
% matrix consisting of the autocorrelation coefficients.
%

  Rtoe=toeplitz(R(1:10));
  ALPH=inv(Rtoe)*R(2:11)';
  ALPH=ALPH';

%
% Derivation of the gain term by matching the energy of the input signal
% with the energy of the linearly predicted samples
%

	G=R(1);
	  for I=1:NP
		G=G-ALPH(I).*R(I+1);
	  end
	G=sqrt(abs(G));

%
% Filtering to obtain the estimated, error, reconstructed, and 
% synthesized signals.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The Estimated Wave
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	b=[0 ALPH];
	a=[1];

% Determines current block of the input signal

	S=X1(1 +(BLOCK - 1)* N:BLOCK * N);
	S=S';

%
% Using b,a filter to obtain estimation
%

	if BLOCK==1
	  [Y,Zf]=filter(b,a,S);        % No initial conditions available for 
	else                           % the first block.
	  [Y,Zf]=filter(b,a,S,Zi);
	end

	X2 = [X2 Y];     % Add current segment to previously filtered segments.
	Zi=Zf;		 % Update the initial conditions.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The Error Wave: use the code for the estimated wave as a model!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%	1.  Determine the vectors a and b for the coefficients of the
%	    Prediction error filter.  Recall that in the Prediction
%	    Error Filter the original signal, X1, is used as the input,
%	    and the output is the error signal, E.
%	
%	2.  Remember to account for the initial conditions that are
%	    needed for the filtering of each block. 
%
%	3.  Keep in mind that you want your FINAL filtered signal to be of the
%	    same length as the orignal signal.
%
%	4.  BE CAREFUL WHEN CHOOSING YOUR VARIABLE NAMES!!
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The Reconstructed Wave
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%	1. Repeat the steps used to produce the ERROR wave.  Recall:
%	   that in the Reconstruction Filter the error signal,E, is used
%	   as the input, and the reconstructed wave, RECON, is the 
%	   output of the filter.
%
%	2. Keep in mind that filtering is being done on only ONE 
%	   SEGMENT of the wave.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The Synthesized Wave
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%	1.  Repeat the steps used to create both the error wave and the
%	    reconstructed wave.  Recall:  the periodic impulse train
%	    excitation is the input to the synthesis filter, and
%	    the synthesized wave, SYNTH, is the output.
%
%	2.  Be sure to include the effects of the gain, G, in your 
%	    determination of the filter coefficients.
%

end

E = E';
X2 = X2';
RECON=RECON';
SYNTH=SYNTH';
