function quant = P_6_9(input,num_bits)
% quant = P_6_9(input,num_bits)
%
% Performs quantization to num_bits bits of the input data.
%
% Note:  assuming input data is in the range -1 to 1.  If this 
% is not the case, scale the input until it is.
%
% Algorithm:  divide by 2, add 0.5:  data
% 	      scaled between 0 and 1.  Multiply by 2^num_bits,
%	      round, divide by 2^num_bits.  Subtract 0.5.  Multiply
%	      by 2.
% 
% Bits:		Levels:
%   1		 +1.0, -1.0
%   2		 +1.0, 0, -1.0
%   3		 +1.0, +0.5, 0, -0.5, -1.0
%   4		 +1.0, +0.75, +0.5, +0.25, 0, -0.25, -0.5, -0.75, -1.0
%
%  and so forth.  Please note that this is not "normal" quantization.
%

Scale_Factor = 2;

Sig = input / Scale_Factor + 0.5;
Sig = round(Sig * (2^(num_bits-1))) / (2^(num_bits-1));
quant = Scale_Factor*(Sig - 0.5);
